@@ -13,7 +13,7 @@ ray_environment = new.env(parent = emptyenv())
   assign("sphere", read_obj(system.file("extdata", "sphere.txt", package="rayvertex")), envir = ray_environment)
   assign("xy_plane", read_obj(system.file("extdata", "xy_plane.txt", package="rayvertex")), envir = ray_environment)
   assign("xz_plane", read_obj(system.file("extdata", "xz_plane.txt", package="rayvertex")), envir = ray_environment)
-  assign("yz_plane", read_obj(system.file("extdata", "xz_plane.txt", package="rayvertex")), envir = ray_environment)
+  assign("yz_plane", read_obj(system.file("extdata", "yz_plane.txt", package="rayvertex")), envir = ray_environment)
   assign("init_time", 0, envir = ray_environment)
   assign("prev_time", 0, envir = ray_environment)
   assign("pkg_loaded", TRUE, envir = ray_environment)
